"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.commandNames = exports.shorthands = exports.rcOptionsTypes = void 0;
exports.cliOptionsTypes = cliOptionsTypes;
exports.help = help;
exports.handler = handler;
const audit_1 = require("@pnpm/audit");
const network_auth_header_1 = require("@pnpm/network.auth-header");
const cli_utils_1 = require("@pnpm/cli-utils");
const config_1 = require("@pnpm/config");
const constants_1 = require("@pnpm/constants");
const error_1 = require("@pnpm/error");
const lockfile_fs_1 = require("@pnpm/lockfile.fs");
const table_1 = require("@zkochan/table");
const chalk_1 = __importDefault(require("chalk"));
const difference_1 = __importDefault(require("ramda/src/difference"));
const pick_1 = __importDefault(require("ramda/src/pick"));
const pickBy_1 = __importDefault(require("ramda/src/pickBy"));
const render_help_1 = __importDefault(require("render-help"));
const fix_1 = require("./fix");
// eslint-disable
const AUDIT_LEVEL_NUMBER = {
    low: 0,
    moderate: 1,
    high: 2,
    critical: 3,
};
const AUDIT_COLOR = {
    low: chalk_1.default.bold,
    moderate: chalk_1.default.bold.yellow,
    high: chalk_1.default.bold.red,
    critical: chalk_1.default.bold.red,
};
const AUDIT_TABLE_OPTIONS = {
    ...cli_utils_1.TABLE_OPTIONS,
    columns: {
        1: {
            width: 54, // = table width of 80
            wrapWord: true,
        },
    },
};
// eslint-enable
const MAX_PATHS_COUNT = 3;
exports.rcOptionsTypes = cliOptionsTypes;
function cliOptionsTypes() {
    return {
        ...(0, pick_1.default)([
            'dev',
            'json',
            'only',
            'optional',
            'production',
            'registry',
        ], config_1.types),
        'audit-level': ['low', 'moderate', 'high', 'critical'],
        fix: Boolean,
        'ignore-registry-errors': Boolean,
    };
}
exports.shorthands = {
    D: '--dev',
    P: '--production',
};
exports.commandNames = ['audit'];
function help() {
    return (0, render_help_1.default)({
        description: 'Checks for known security issues with the installed packages.',
        descriptionLists: [
            {
                title: 'Options',
                list: [
                    {
                        description: 'Add overrides to the package.json file in order to force non-vulnerable versions of the dependencies',
                        name: '--fix',
                    },
                    {
                        description: 'Output audit report in JSON format',
                        name: '--json',
                    },
                    {
                        description: 'Only print advisories with severity greater than or equal to one of the following: low|moderate|high|critical. Default: low',
                        name: '--audit-level <severity>',
                    },
                    {
                        description: 'Only audit "devDependencies"',
                        name: '--dev',
                        shortAlias: '-D',
                    },
                    {
                        description: 'Only audit "dependencies" and "optionalDependencies"',
                        name: '--prod',
                        shortAlias: '-P',
                    },
                    {
                        description: 'Don\'t audit "optionalDependencies"',
                        name: '--no-optional',
                    },
                    {
                        description: 'Use exit code 0 if the registry responds with an error. Useful when audit checks are used in CI. A build should fail because the registry has issues.',
                        name: '--ignore-registry-errors',
                    },
                ],
            },
        ],
        url: (0, cli_utils_1.docsUrl)('audit'),
        usages: ['pnpm audit [options]'],
    });
}
async function handler(opts) {
    const lockfileDir = opts.lockfileDir ?? opts.dir;
    const lockfile = await (0, lockfile_fs_1.readWantedLockfile)(lockfileDir, { ignoreIncompatible: true });
    if (lockfile == null) {
        throw new error_1.PnpmError('AUDIT_NO_LOCKFILE', `No ${constants_1.WANTED_LOCKFILE} found: Cannot audit a project without a lockfile`);
    }
    const include = {
        dependencies: opts.production !== false,
        devDependencies: opts.dev !== false,
        optionalDependencies: opts.optional !== false,
    };
    let auditReport;
    const getAuthHeader = (0, network_auth_header_1.createGetAuthHeaderByURI)({ allSettings: opts.rawConfig, userSettings: opts.userConfig });
    try {
        auditReport = await (0, audit_1.audit)(lockfile, getAuthHeader, {
            agentOptions: {
                ca: opts.ca,
                cert: opts.cert,
                httpProxy: opts.httpProxy,
                httpsProxy: opts.httpsProxy,
                key: opts.key,
                localAddress: opts.localAddress,
                maxSockets: opts.maxSockets,
                noProxy: opts.noProxy,
                strictSsl: opts.strictSsl,
                timeout: opts.fetchTimeout,
            },
            include,
            lockfileDir,
            registry: opts.registries.default,
            retry: {
                factor: opts.fetchRetryFactor,
                maxTimeout: opts.fetchRetryMaxtimeout,
                minTimeout: opts.fetchRetryMintimeout,
                retries: opts.fetchRetries,
            },
            timeout: opts.fetchTimeout,
            virtualStoreDirMaxLength: opts.virtualStoreDirMaxLength,
        });
    }
    catch (err) { // eslint-disable-line
        if (opts.ignoreRegistryErrors) {
            return {
                exitCode: 0,
                output: err.message,
            };
        }
        throw err;
    }
    if (opts.fix) {
        const newOverrides = await (0, fix_1.fix)(opts.dir, auditReport);
        if (Object.values(newOverrides).length === 0) {
            return {
                exitCode: 0,
                output: 'No fixes were made',
            };
        }
        return {
            exitCode: 0,
            output: `${Object.values(newOverrides).length} overrides were added to package.json to fix vulnerabilities.
Run "pnpm install" to apply the fixes.

The added overrides:
${JSON.stringify(newOverrides, null, 2)}`,
        };
    }
    const vulnerabilities = auditReport.metadata.vulnerabilities;
    const ignoredVulnerabilities = {
        low: 0,
        moderate: 0,
        high: 0,
        critical: 0,
    };
    const totalVulnerabilityCount = Object.values(vulnerabilities)
        .reduce((sum, vulnerabilitiesCount) => sum + vulnerabilitiesCount, 0);
    const ignoreGhsas = opts.rootProjectManifest?.pnpm?.auditConfig?.ignoreGhsas;
    if (ignoreGhsas) {
        // eslint-disable-next-line @typescript-eslint/naming-convention
        auditReport.advisories = (0, pickBy_1.default)(({ github_advisory_id, severity }) => {
            if (!ignoreGhsas.includes(github_advisory_id)) {
                return true;
            }
            ignoredVulnerabilities[severity] += 1;
            return false;
        }, auditReport.advisories);
    }
    const ignoreCves = opts.rootProjectManifest?.pnpm?.auditConfig?.ignoreCves;
    if (ignoreCves) {
        auditReport.advisories = (0, pickBy_1.default)(({ cves, severity }) => {
            if (cves.length === 0 || (0, difference_1.default)(cves, ignoreCves).length > 0) {
                return true;
            }
            ignoredVulnerabilities[severity] += 1;
            return false;
        }, auditReport.advisories);
    }
    if (opts.json) {
        return {
            exitCode: totalVulnerabilityCount > 0 ? 1 : 0,
            output: JSON.stringify(auditReport, null, 2),
        };
    }
    let output = '';
    const auditLevel = AUDIT_LEVEL_NUMBER[opts.auditLevel ?? 'low'];
    let advisories = Object.values(auditReport.advisories);
    advisories = advisories
        .filter(({ severity }) => AUDIT_LEVEL_NUMBER[severity] >= auditLevel)
        .sort((a1, a2) => AUDIT_LEVEL_NUMBER[a2.severity] - AUDIT_LEVEL_NUMBER[a1.severity]);
    for (const advisory of advisories) {
        const paths = advisory.findings.map(({ paths }) => paths).flat();
        output += (0, table_1.table)([
            [AUDIT_COLOR[advisory.severity](advisory.severity), chalk_1.default.bold(advisory.title)],
            ['Package', advisory.module_name],
            ['Vulnerable versions', advisory.vulnerable_versions],
            ['Patched versions', advisory.patched_versions],
            [
                'Paths',
                (paths.length > MAX_PATHS_COUNT
                    ? paths
                        .slice(0, MAX_PATHS_COUNT)
                        .concat([
                        `... Found ${paths.length} paths, run \`pnpm why ${advisory.module_name}\` for more information`,
                    ])
                    : paths).join('\n\n'),
            ],
            ['More info', advisory.url],
        ], AUDIT_TABLE_OPTIONS);
    }
    return {
        exitCode: output ? 1 : 0,
        output: `${output}${reportSummary(auditReport.metadata.vulnerabilities, totalVulnerabilityCount, ignoredVulnerabilities)}`,
    };
}
function reportSummary(vulnerabilities, totalVulnerabilityCount, ignoredVulnerabilities) {
    if (totalVulnerabilityCount === 0)
        return 'No known vulnerabilities found\n';
    return `${chalk_1.default.red(totalVulnerabilityCount)} vulnerabilities found\nSeverity: ${Object.entries(vulnerabilities)
        .filter(([auditLevel, vulnerabilitiesCount]) => vulnerabilitiesCount > 0)
        .map(([auditLevel, vulnerabilitiesCount]) => AUDIT_COLOR[auditLevel](`${vulnerabilitiesCount} ${auditLevel}${ignoredVulnerabilities[auditLevel] > 0 ? ` (${ignoredVulnerabilities[auditLevel]} ignored)` : ''}`))
        .join(' | ')}`;
}
//# sourceMappingURL=audit.js.map